BINDING_HEADER_BIGGERBATTLESHOUT = "BigBattleShout";

function BBS_ShouldShout() return shouldShout end
function BBS_HasSwapQueued() return equipQueued end

local _, playerClass = UnitClass("player");
local playerName = UnitName("player");
local tooltip = CreateFrame("GameTooltip", "BBSTooltip", nil, "GameTooltipTemplate");
local BBS_Frame = CreateFrame("Frame");
local BBSEvents = {}

local WrathPieces = {
  [1] = 16963,
  [3] = 16961,
  [5] = 16966,
  [6] = 16960,
  [7] = 16962,
  [8] = 16965,
  [9] = 16959,
  [10] = 16964,
};
local ShoutAP = {
  [11551] = 193,
  [25289] = 232,
};
local BBSrank;
local BBSap;

local setPieces = {};
local baggedItems = {};
local baggedItemsBackup = {};
local BBSTicker;
local hasBagged = false;
local BBSGearTicker;

local equipQueued = false;
local isInCombat = false;
local ICDTimestamp = 0;

local shouldShout = false;

function BBS_Initialize()
  if(GetSpellBookItemName(1, "spell") ~= nil) then
    BBS_CalculateBaseAP();
    BBS_SetWrathPieces();
    BBS_Frame:SetScript("OnUpdate", nil);
  end
end

function BBS_CalculateBaseAP()
  if(not BBSrank) then BBS_CreateTooltip() end
  local _ , _, _, _, currentRank, maxRank, _, _ = GetTalentInfo(2, 8);
  BBSap = math.floor(ShoutAP[BBSrank] * (1 + 0.05 * currentRank)) + 28;
end

function BBS_CreateTooltip()
  local _, _, _, _, _, _, rank = GetSpellInfo("Battle Shout");
  BBSrank = rank;
  tooltip:SetSpellByID(BBSrank);
  tooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
end

function BBS_GetTooltipAP()
  if(not BBSrank) then BBS_CreateTooltip() end
  tooltip:SetSpellByID(BBSrank);
  local costText = BBSTooltipTextLeft4:GetText();
  return string.match(costText, "yards by ([0-9]+)");
end

function BBS_HasBonusEquipped()
  local equipped = 0;
  for slotID, wrathID in pairs(WrathPieces) do
    if(wrathID == GetInventoryItemID("player", slotID)) then
      equipped = equipped + 1;
    end
  end
  return equipped >= 3, equipped;
end

function BBS_EquipBagged()
  for k, v in pairs(baggedItems) do
    if(isInCombat) then return end
    EquipItemByName(v);
    baggedItems[k] = nil;
  end
  equipQueued = false;
  hasBagged = false;
  BBS_ToggleTicker(false);
  BBSGearTicker = C_Timer.After(0.2, function()
    local unequipped = 0;
    for slot, item in pairs(baggedItemsBackup) do
      unequipped = unequipped + 1;
      local equippedName = GetItemInfo(GetInventoryItemLink("player", slot));
      if(equipped == item) then
        baggedItemsBackup[slot] = nil;
        unequipped = unequipped - 1;
      end
    end
    if(unequipped ~= 0) then
      baggedItems = baggedItemsBackup;
      equipQueued = true;
      hasBagged = true;
      print("BAA: Gear swap queued.");
    end
    baggedItemsBackup = {};
  end);
end

function BBS_EquipWrath()
  for slot, name in pairs(setPieces) do
    if(isInCombat) then return end
    local equippedName = GetItemInfo(GetInventoryItemLink("player", slot));
    baggedItems[slot] = equippedName;
    EquipItemByName(name);
  end
  hasBagged = true;
  baggedItemsBackup = baggedItems;
  equipQueued = false;
  shouldShout = true;
  BBS_ToggleTicker(true);
end

function BBS_SetWrathPieces()
  local foundPieces = {};
  local priority = {
    [1] = 9,
    [2] = 7,
    [3] = 3,
    [4] = 10,
    [5] = 5,
    [6] = 6,
    [7] = 8,
    [8] = 1,
  };
  local pieceCount = 0;
  for k, v in pairs(priority) do
    local id = WrathPieces[v];
    if(GetItemCount(id) > 0) then
      local name = GetItemInfo(id);
      foundPieces[v] = name;
      pieceCount = pieceCount + 1;
    end
    if(pieceCount == 3) then
      break
    end
  end
  if(pieceCount == 3) then
    setPieces = foundPieces;
  end
end

function BBS_EquipQueued()
  if(not hasBagged) then
    BBS_EquipWrath();
    print("BAA: Equipping 3 set.");
  else
    BBS_EquipBagged();
    print("BAA: Re-equipping.");
    BBS_ToggleTicker(false);
  end
  equipQueued = false;
end

function BBS_QueueSwap()
  if(ICDTimestamp < GetTime()) then
    if(equipQueued) then
      equipQueued = false;
      print("BAA: Gear queue canceled.");
      return;
    end
    if(isInCombat) then
      equipQueued = true;
      print("BAA: Gear swap queued.");
    else
      BBS_EquipQueued();
    end
    ICDTimestamp = GetTime() + 0.5;
  end
end

function BBS_ToggleTicker(state)
  if(state) then
    BBSTicker = C_Timer.NewTicker(0.1, function()
      local tooltipAP = BBS_GetTooltipAP();
      if(tonumber(tooltipAP) > BBSap) then
        shouldShout = true;
        BBSTicker:Cancel();
      end
    end);
  else
    if(BBSTicker ~= nil) then
      BBSTicker:Cancel();
    end
  end
end

function BBSEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, eventType, _, sourceGUID,
      sourceName, _, _, destGUID,
      destName, _, _, spellId,
      spellName, _, auraType, _,
      _,isCrit,_ = CombatLogGetCurrentEventInfo();
  if(eventType == "SPELL_CAST_SUCCESS") then
    if(spellName == "Battle Shout" and sourceName == playerName and shouldShout) then
      shouldShout = false;
      BBS_QueueSwap();
    end
  end
end

function BBSEvents:PLAYER_REGEN_ENABLED(...)
  isInCombat = false;
  if(equipQueued) then
    BBS_EquipQueued();
  end
end

function BBSEvents:PLAYER_REGEN_DISABLED(...)
  isInCombat = true;
end

function BBSEvents:CHARACTER_POINTS_CHANGED(...)
  BBS_CalculateBaseAP();
end

local BagTicker;
local lastBagUpdate = 0;
function BBSEvents:BAG_UPDATE(...)
  if(not IsInInstance()) then
    lastBagUpdate = GetTime();
    if(BagTicker == nil) then
      BagTicker = C_Timer.NewTicker(2, function()
        if(not isInCombat and GetTime() - lastBagUpdate > 5) then
          BBS_SetWrathPieces();
          BagTicker:Cancel();
        end
      end);
    end
  end
end

if(playerClass ~= "WARRIOR" or UnitLevel("player") < 60) then
  return
else
  BBS_Frame:SetScript("OnUpdate", BBS_Initialize);
  BBS_Frame:SetScript("OnEvent", function(self, event, ...)
    BBSEvents[event](self, ...);
  end);
  for k, v in pairs(BBSEvents) do
    BBS_Frame:RegisterEvent(k);
  end
end
